require('coffee-script/register');

var server = require('../node_modules/karma/lib/server');
var _ = require('../node_modules/lodash/lodash.js');
var path= require('path');

var basePattern = path.resolve(__dirname + '/../');

var preprocessors = {};

var requirejsConfig = {
  baseUrl: "/base",
  deps: [],
  callback: function() {
    return window.__karma__.start();
  },
  paths: {},
  shim: {},
  config: {}
};

var mergeConfig = {
  //configFile: '/Users/roberto/Desktop/karmajs-bootstrap/karma.conf.coffee'
  action: 'start'
  ,browsers: ['Firefox']
  ,frameworks: ['mocha', 'requirejs', 'bootstrap']
  ,files: [
    {pattern: 'components/jquery/dist/jquery.min.js', included: false, served: true, requirejs: {name: 'jquery'}},
    {pattern: 'components/lodash/dist/lodash.min.js', included: false, served: true, requirejs: {name: 'lodash'}},
    {pattern: 'components/requirejs-text/text.js', included: false, served: true, requirejs: {name: 'text'}},
    {pattern: 'components/chai/chai.js', included: false, served: true, requirejs: {name: 'chai'}},
    {pattern: 'components/merge.min.js', included: false, served: true, requirejs: {name: 'merge'}}
  ]
  ,client: {
    requirejs: requirejsConfig,
    args: [],
    useIframe: true,
    captureConsole: true
  }
  ,yamlPreprocessor:{
    options:{
      sourceMap: true
    }
  }
  ,coffeePreprocessor:{
    options:{
      sourceMap: true
    }
  }
  ,preprocessors: {
    'test/**/*.coffee': 'coffee'
  }
  ,coverageReporter: {}
};

var defaultConfig = {
  basePath: './'
  ,exclude: []
  ,colors: true
  ,autoWatch: true
  ,autoWatchBatchDelay: 250
  ,reporters: ['dots', 'coverage']
  ,singleRun: true
  ,captureTimeout: 60000
  ,proxies: {}
  ,proxyValidateSSL: true
  ,preprocessors: {}
  ,urlRoot: '/'
  ,reportSlowerThan: 0
  ,transports: ['websocket', 'flashsocket', 'xhr-polling', 'jsonp-polling']
  ,plugins: [
    'karma-bootstrap-framework',
    "karma-coffee-preprocessor",
    "karma-commonjs",
    "karma-coverage",
    "karma-firefox-launcher",
    "karma-mocha",
    "karma-ng-html2js-preprocessor",
    "karma-requirejs",
    "karma-yaml-preprocessor",
    'karma-*'
  ]
  ,browserDisconnectTimeout: 2000
  ,browserDisconnectTolerance: 2
  ,browserNoActivityTimeout: 10000
};

var handleFileMerge = function(configObj, settingObj) {
  if(_.isArray(configObj)){
    return settingObj.concat(configObj);
  }else{
    return settingObj;
  }
}

var handleObjectMerge = function(configObj, settingObj) {
  if(_.isPlainObject(configObj)){
    return _.merge(configObj, settingObj);
  }else{
    return settingObj;
  }
}

var handleConfigType = function(result, value, key) {
  if(_.has(result, key)){
    switch(key){
      case 'preprocessors':
        result[key] = handleObjectMerge(result[key], value);
        return result;
        break;

      case 'coverageReporter':
        result[key] = handleObjectMerge(result[key], value);
        return result;
        break;

      case 'client':
        result[key] = handleObjectMerge(result[key], value);
        return result;
        break;

      case 'coffeePreprocessor':
        result[key] = handleObjectMerge(result[key], value);
        return result;
        break;

      case 'yamlPreprocessor':
        result[key] = handleObjectMerge(result[key], value);
        return result;
        break;

      case 'files':
        result[key] = handleFileMerge(result[key], value);
        return result;
        break;
      default:
        result[key] = value;
        return result;
        break;
    }
  }else{
    result[key] = value;
    return result;
  }
}

var defaultsDeep = function(config, setting) {
  var mapper = _.reduce(setting, function(result, value, key) {
    return handleConfigType(result, value, key);
  }, config);
  return mapper;
};

var getRequireJsFiles = function(config) {
  if(!config.client || !config.client.requirejs){ return config; };
  config.basePath = config.basePath || '';

  var len = config.files.length
  var replaceBasePath = path.resolve(basePattern, config.basePath);
  var item, requireJsPathFile, requireJsPath;
  for (var i = 0; i < len; i++) {
    item = config.files[i];
    if(item.requirejs && item.pattern && _.isPlainObject(item.requirejs)){
      requireJsPathFile = path.resolve(basePattern, item.pattern).toString();
      requireJsPathFile = requireJsPathFile.replace(replaceBasePath, '/base');
      requireJsPathFile = requireJsPathFile.replace(/\.js$/, '');
      requireJsPath = config.client.requirejs.paths;

      if(_.has(item.requirejs, 'name')){
        config.client.requirejs.paths[item.requirejs.name] = requireJsPathFile
      }

      if(_.has(item.requirejs, 'shim') && config.client.requirejs.shim){
        config.client.requirejs.shim[requireJSPathName] = item.requirejs.shim
      }

      if(_.has(item.requirejs, 'config') && config.client.requirejs.config){
        config.client.requirejs.config[requireJSPathName] = item.requirejs.config
      }
      delete item.requirejs;
    }
  }
  return config;
};

var getRequireJsObject = function(config) {
  if(!config.client.requirejs){
    config.client.requirejs = {};
  }
  for (var key in config.requireJsConfig) {
    if(config.requireJsConfig.hasOwnProperty(key)){
      if(!config.client.requirejs[key]){
        config.client.requirejs[key] = {};
      }
      switch(key){
        case 'callback':
          //do nothing for now
          break;

        case 'deps':
          if(_.isArray(config.requireJsConfig[key])){
            config.client.requirejs[key].concat(config.requireJsConfig[key]);
          }
          break;

        default:
          config.client.requirejs[key] = config.requireJsConfig[key];
          break;
      }
    }
  }
  return config;
};

exports = module.exports = {
  start: function(userConfig){
    var config = defaultsDeep(_.assign(defaultConfig, userConfig), mergeConfig);
    config = getRequireJsObject(config);
    config = getRequireJsFiles(config);

    server.start(config);
  }
}
